#include "VisibleForm.h"

#ifndef _DEBUG_
#define _DEBUG_
#endif

// local variables
#define Factor100	100.0	//factor
#define Factor10	10.0	//factor2
#define Factor1000	1000.0	//factor3
#define Factor10000 10000.0	//factor4

//Global Variables
int			prevWidth;			//save width
int			prevHeight;			//save height
QDir		*dir;
QString		currDir;
QString		str;
QString		home;
const int	main_wind_height = 698;
const int	auxy_wind_height = 148;

VisibleForm::VisibleForm ( QWidget* parent, const char* name, bool modal, WFlags fl) 
        :MainForm ( parent, name, fl )
{
    bool l_modal = modal;
    l_modal = false;

    lb_status_text->setText("Initializing application");
    printf("Initializing application\n");
    mo = new MathObject ( this );

    vp = new OGLWidgetMain ( frm_OGLScreen1, "ViewPane", this );
    vp->setGeometry ( 1,1,main_wind_height,main_wind_height);
    //vp->setFocusPolicy(QWidget::StrongFocus);
    //vp->setFocus();
    vp->setMouseTracking(TRUE);
	vp->installEventFilter(this);

    fwin_vp = new OGLWidgetFloating ( frm_OGLScreen2, "Floating Viewing Pane", this );
    fwin_vp->setGeometry ( 1,1,auxy_wind_height,auxy_wind_height );
	fwin_vp->installEventFilter(this);
    
	fwin2_vp = new OGLWidgetFloating2 ( frm_OGLScreen3, "Floating Viewing Pane", this );
    fwin2_vp->setGeometry ( 1,1,auxy_wind_height,auxy_wind_height );
	//fwin2_vp->setMouseTracking(TRUE);
    fwin2_vp->installEventFilter(this); 

#ifdef _DEBUG_
    home = QDir( "./data/index.htm" ).absPath();
#else
    home = "../data/index.htm";
#endif

	// texture related
	dir = new QDir();    
	
	initForm();
    swapFrames = 0;
    swapFrames2= 0;
	//calbak_Swap_Frames_2();

	//this->showMaximized();
    lb_status_text->setText("Application loaded and ready...");
}
VisibleForm::~VisibleForm ( )
{
    fprintf(stderr,"cleanup: VisibleForm\n");
    zap(mo);
    zap(vp);
    zap(fwin_vp);
}


//# INITIALIZE
void VisibleForm::initForm										( void )
{
#ifdef _DBG_
	////printf("inside initForm()\n");
#endif
    double	val		= 0.0;
    int		a,b,r;
	int		texVal	= 0;

    //connect(lb_status_text, SIGNAL(), this, SLOT() );

	//printf("whichCurve = %d, OBJSPLINE = %d, OBJKNOT = %d\n",mo->whichCurve, OBJSPLINE, OBJKNOT);
	/* global initializations */
    rb_draw_curve_knots->setChecked(TRUE);
    rb_frame_noframe->setEnabled(FALSE);
	gbox_graph_elements->show();
	rb_color_solid->setChecked(TRUE);

    if(mo->qshowReflectedPoints)
        cb_qmap_reflect->setChecked(TRUE);

    rb_show_quatcomp_w->setChecked(TRUE);
    rb_frame_pt_quat->setChecked(TRUE);
    if(mo->showLabels)
        cb_show_labels->setChecked(TRUE);
    if(mo->objectID	== OBJCURVE)
        rb_draw_curve->setChecked(TRUE);
    if(mo->whichCurve == OBJSPLINE)
        rb_draw_curve_edit->setChecked(TRUE);
    if(mo->whichCurve == OBJKNOT)
        rb_draw_curve_knots->setChecked(TRUE);
    if(mo->selectFrame_FS)
        cb_qmap_f->setChecked(TRUE);
    else
        cb_qmap_f->setChecked(FALSE);
    if(mo->selectFrame_PT)
        cb_qmap_pt->setChecked(TRUE);
    else
        cb_qmap_pt->setChecked(FALSE);
    if(mo->selectFrame_GR)
        cb_qmap_gr->setChecked(TRUE);
    else
        cb_qmap_gr->setChecked(FALSE);

    if(mo->rshowFace)
        cb_qmap_face->setChecked(TRUE);
    else
        cb_qmap_face->setChecked(FALSE);

    if(mo->rshowEdge)
        cb_qmap_ring_edge->setChecked(TRUE);
    else
        cb_qmap_ring_edge->setChecked(FALSE);
    
        //quaternion-map initializations
    if(mo->qshowReflectedPoints)
        cb_qmap_reflect->setChecked(TRUE);
    
    
    switch(mo->whichSpline) {
        case BEZIER:
            rb_spline_bz->setChecked(TRUE);
            break;
        case BSPLINE:
            rb_spline_bspline->setChecked(TRUE);
            break;
        case CATROM:
            rb_spline_catrom->setChecked(TRUE);
            break;
        case NOSPLINE:
            rb_spline_piece_wise->setChecked(TRUE);
            break;
        default:
            break;
    }
        
        /* KNOT Control initializations */
        // range for knot parameter values 
    slider_knot_m->setMaxValue(10);
    slider_knot_n->setMaxValue(10);
        //slider_knot_a->setMinValue(1);
    slider_knot_a->setMaxValue(1000);
        //slider_knot_b->setMinValue(1);
    slider_knot_b->setMaxValue(1000);
    slider_knot_radius->setMinValue(0);
    slider_knot_radius->setMaxValue(200);
    slider_pt_start_angle->setMaxValue(1300);
    slider_pt_start_angle->setMinValue(-5);
    slider_pt_start_angle_2->setMaxValue(1300);
    slider_pt_start_angle_2->setMinValue(-5);
    slider_ring_number->setMaxValue(4);
    slider_ring_number->setMinValue(0);
    
    str.sprintf("%s %d","m=", mo->m);
    lb_knot_m->setText(str);
    slider_knot_m->setValue(mo->m);
    
    str.sprintf("%s %d","n=", mo->n);
    lb_knot_n->setText(str);
    slider_knot_n->setValue(mo->n);

    val=mo->a;
    a = (int)(val*Factor1000);
    str.sprintf("%s %0.2f","a=",val);
    lb_knot_a->setText(str);
    slider_knot_a->setValue(a);

    val=mo->b;
    b = (int)(val*Factor1000);
    str.sprintf("%s %0.2f","b=",val);
    lb_knot_b->setText(str);
    slider_knot_b->setValue(b);
    
    val=mo->r;
    r = (int)(val*Factor100);
    str.sprintf("%s %0.2f","r=",val);
    lb_knot_r->setText(str);
    slider_knot_radius->setValue(r);
    
    val = mo->startTheta;
    str.sprintf("%s %0.2f","PT: Start angle=",val);
    lb_knot_start_angle->setText(str);
	//copy
    val = mo->startTheta;
    str.sprintf("%s %0.2f","PT: Start angle=",val);
    lb_knot_start_angle_2->setText(str);
	//

	//rings
    str.sprintf("%s %.0f %","Number of Rings= ", (((double)mo->numOfRings)/4.0)*100.0);
    lb_ring_number->setText(str);
    slider_ring_number->setValue(mo->numOfRings);
	
	str.sprintf("%s %0.3f","Angle=",mo->rquatRingAngle);
    lb_dial_quat_ring_angle->setText(str);
    
	//frame type
    if(mo->whichFrame == FRENETSERRET)
        rb_frame_f->setChecked(TRUE);
    else if(mo->whichFrame == PARALLELTRANSPORT)
        rb_frame_pt->setChecked(TRUE);

    if(mo->selectFrame_FS)
        cb_qmap_f->setChecked(TRUE);
    if(mo->selectFrame_PT)
        cb_qmap_pt->setChecked(TRUE);
    if(mo->selectFrame_GR)
        cb_qmap_gr->setChecked(TRUE);
	

	//@ Quaternion Map widgets
    val=mo->qmaptubeThickness;
	str.sprintf("%s %0.2f","Tube Thickness: ",val);
    lb_qmap_tubethickness->setText(str);
    slider_qmap_tubethickness->setValue((int)(val*Factor100));

    val=mo->tubeThickness;
    str.sprintf("%s %0.2f","Tube Thickness: ",val);
    lb_curve_tubethickness->setText(str);
    slider_curve_tubethickness->setValue((int)(val*Factor100));

	//@ Curve widgets
    slider_spline_t->setMaxValue(100);
    slider_spline_t->setMinValue(1);
    mo->param_spline_increment = 0.02;
    slider_spline_t->setValue( (int)(Factor100*mo->param_spline_increment) );
    str.sprintf("%s %0.2f","t: ",	  mo->param_spline_increment );
    lb_num_pnts->setText(str);
    

	//@ Surface widgets
	if(mo->objectID==OBJCURVE) {
		gbox_select_2->setEnabled(FALSE);
	}

    slider_surface_param_a->setMaxValue(1000);
    slider_surface_param_b->setMaxValue(1000);  
	slider_surface_param_r->setMaxValue(200);
	str.sprintf("%s %d","u=", mo->u);
    lb_surface_u->setText(str);
    slider_surface_u->setValue(mo->u);
    
    str.sprintf("%s %d","v=", mo->v);
    lb_surface_v->setText(str);
    slider_surface_v->setValue(mo->v);

    val=mo->param_a;
    a = (int)(val*Factor1000);
    str.sprintf("%s %0.2f","a=",val);
    lb_surface_param_a->setText(str);
    slider_surface_param_a->setValue(a);

    val=mo->param_b;
    b = (int)(val*Factor1000);
    str.sprintf("%s %0.2f","b=",val);
    lb_surface_param_b->setText(str);
    slider_surface_param_b->setValue(b);

    val=mo->param_r;
    r = (int)(val*Factor100);
    str.sprintf("%s %0.2f","r=",val);
    lb_surface_r->setText(str);
    slider_surface_param_r->setValue(r);

	//@ Optimization widgets
	//slider_opt_set_increment->setMaxValue(1000);
	//slider_opt_set_increment->setMinValue(1);
    str.sprintf("%s %.3f","= ", mo->opt_EPSILON);
    lb_opt_increment->setText(str);
	//opt_epsilon = (int)(mo->opt_EPSILON*Factor1000);
    //slider_opt_set_increment->setValue(opt_epsilon);

	/* texture options */

	//texture mgmt
	texUpdateList();
	cbox_loadtexture->setEnabled(FALSE);
    val=mo->texParam;
    texVal = (int)(val*Factor10);
    str.sprintf("%s %0.1f","Texture param= ",val);
    lb_slider_texture_control->setText(str);
    slider_texture_control->setValue(texVal);

	//graph options
	if(mo->gshowQdist)
		cb_graph_qdist->setChecked(TRUE);
	if(mo->gshowQdotprod)
		cb_graph_qdotprod->setChecked(TRUE);
	if(mo->gshowQtangacc)
		cb_graph_qtangacc->setChecked(TRUE);
	if(mo->gshowCcurvature)
		cb_graph_qcurvature->setChecked(TRUE);
	if(mo->gshowCtorsion)
		cb_graph_qtorsion->setChecked(TRUE);
	
    
    //Loading Meshview files
    //@ curvesUpdateList();
    cbox_curve_list->setEnabled(FALSE);

	calbak_DrawingOptions();
	
	stateChange();
	
	//install filters
	this->installEventFilter(this);

    //set palette images
    cbox_palette->insertItem(QPixmap::fromMimeSource("palette1.png"),"Palette 1");
    cbox_palette->insertItem(QPixmap::fromMimeSource("palette2.png"),"Palette 2");
    cbox_palette->insertItem(QPixmap::fromMimeSource("palette3.png"),"Palette 3");
    cbox_palette->insertItem(QPixmap::fromMimeSource("palette4.png"),"Palette 4");
    //b_clear->setPixmap( QPixmap(i));


    //hide stuff
    gbox_hide_graph->hide();
    gbox_hide_optimization_options->hide();
    gbox_hide_quat_ring_options->hide();
    gbox_hide_select_type->hide();
    gbox_hide_quatmap_curve_frame_options->hide();
    b_clear->hide();

    rb_draw_curve_edit->setChecked(FALSE);
    rb_draw_curve_knots->setChecked(TRUE);
    calbak_DrawingOptions_CurveType();
    
}
bool VisibleForm::eventFilter									( QObject *target, QEvent *event ) 
{	
	//mo->sessionStr = "";
	//if(target == fwin_vp && (event->type() == QEvent::MouseButtonPress) ) {
	//	QMouseEvent *mouseEvent = (QMouseEvent*) event;
	//	//printf("Caught mouse click: x= %d, y= %d\n",mouseEvent->x(),mouseEvent->y() );
	//	mo->sessionStr =	QString("mouse click:") + QString("\tx = ") + 
	//						QString(QString::number(mouseEvent->x())) + QString("\ty = ") + 
	//						QString(QString::number(mouseEvent->y())) + 
	//						QString("\n");
	//}
	//else if(target == fwin_vp && (event->type() == QEvent::MouseMove) ) {
	//	QMouseEvent *mouseEvent = (QMouseEvent*) event;
	//	//printf("Caught mouse click: x= %d, y= %d\n",mouseEvent->x(),mouseEvent->y() );
	//	mo->sessionStr =	QString("mouse move:") + QString("\tx = ") + 
	//						QString(QString::number(mouseEvent->x())) + QString("\ty = ") + 
	//						QString(QString::number(mouseEvent->y())) + 
	//						QString("\n");
	//}
	//else if( target == btn_ok && (event->type() == QEvent::MouseButtonPress) ) {
	//	//printf("OKAY clicked...\n");
	//	mo->sessionStr =	QString("button click:\tOKAY") + QString("\n");
	//}
	//else if( target == rb_choice_one && (event->type() == QEvent::MouseButtonPress) ) {
	//	//printf("RB-ONE clicked...\n");
	//	mo->sessionStr =	QString("radio button click:\tCHOICE-ONE") + QString("\n");
	//}
	//else if( target == rb_choice_two && (event->type() == QEvent::MouseButtonPress)) {
	//	//printf("RB-TWO clicked...\n");
	//	mo->sessionStr =	QString("radio button click:\tCHOICE-TWO") + QString("\n");
	//}
	//else if( target == btn_bye && (event->type() == QEvent::MouseButtonPress)) {
	//	//printf("EXIT clicked...\n");
	//	mo->sessionStr =	QString("button click:\tEXIT") + QString("\n");

	//}
	//else 
	if( event->type() == QEvent::KeyPress )  {
		QKeyEvent *keyEvent = (QKeyEvent*) event;
		//printf("Key pressed...\n");
		switch(keyEvent->key()) {
			case Key_Control:
				mo->flag_keypress_CTRL = 1;
				break;
			case Key_Shift:
				mo->flag_keypress_SHIFT = 1;
				break;
			case Key_Alt:
				mo->flag_keypress_ALT = 1;
				break;
			default:
				break;
		}
		//mo->sessionStr = QString("key pressed:") + QString("\tkey = ") + QString::number(keyEvent->key()) + QString("\n");
		return false;
	}
	else if( event->type() == QEvent::KeyRelease ) {

		mo->flag_keypress_ALT	= 0;
		mo->flag_keypress_CTRL	= 0;
		mo->flag_keypress_SHIFT	= 0;
	}
	return false;
}

void VisibleForm::timerEvent									( QTimerEvent *e )
{
	int i = 0;
	if(mo->whichCurve == OBJSPLINE)
		i = mo->numPoints-3;
	else if(mo->whichCurve == OBJKNOT)
		i = mo->numPoints-1;

	//if(mo->opt_INDEX == mo->numPoints-2) {
	//	//mo->flag_opt_continue = 0;
	//	mo->opt_INDEX = 1;
	//}
    if(mo->pt_flag_terminate) {
        mo->flag_opt_continue = 0;
    }
	if(!mo->flag_opt_continue) {
		killTimer( e->timerId() );
		return;
	}
	mo->opt_INDEX += 1;
    //mo->rquatRingAngle += 0.5;
	//@ for repeating
	//if(mo->opt_INDEX > i)
	//	mo->opt_INDEX = 1;
	//mo->trackingRing = mo->opt_INDEX;
	//updateQuaternionRingElements();
	mo->optimization();
	notifyOGLWidgets();
}
void VisibleForm::stateChange									( void ) 
{	
	if(mo->showOnScreenHelp)
		cb_show_screen_text->setChecked(TRUE);
	else
		cb_show_screen_text->setChecked(FALSE);
    
    if(mo->whoCommandsMainWin == MAIN_WINDOW) {
		
        cb_show_curve->setText(QString("Curve (c)"));
        cb_edge->setEnabled(TRUE);
        cb_face->setText(QString("Face (f)"));
        cb_frames->setEnabled(TRUE);
        cb_show_gap->setEnabled(TRUE);
        cb_qmap_reflect->setEnabled(FALSE);
        cb_show_sphere->setEnabled(FALSE);
        cb_qmap_rings->setEnabled(FALSE);
        gbox_graph_elements->show();

        if(mo->showAxes)
            cb_axes->setChecked(TRUE);
        else
            cb_axes->setChecked(FALSE);

        if(mo->showEdge)
            cb_edge->setChecked(TRUE);
        else
            cb_edge->setChecked(FALSE);

        if(mo->showFrame)
            cb_frames->setChecked(TRUE);
        else
            cb_frames->setChecked(FALSE);

        if(mo->showFace)
            cb_face->setChecked(TRUE);
        else
            cb_face->setChecked(FALSE);

        if(mo->showCurve)
            cb_show_curve->setChecked(TRUE);
        else
            cb_show_curve->setChecked(FALSE);

        if(mo->showGap)
            cb_show_gap->setChecked(TRUE);
        else
            cb_show_gap->setChecked(FALSE);
        
        switch(mo->whichSpline) {
            case BEZIER:
                rb_spline_bz->setChecked(TRUE);
                break;
            case BSPLINE:
                rb_spline_bspline->setChecked(TRUE);
                break;
            case CATROM:
                rb_spline_catrom->setChecked(TRUE);
                break;
            case NOSPLINE:
                rb_spline_piece_wise->setChecked(TRUE);
                break;
            default:
                break;
        }

    }
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1) {
		
        cb_show_curve->setText(QString("Curve (c)"));
        cb_edge->setEnabled(TRUE);
        cb_face->setText(QString("Face (f)"));
        cb_frames->setEnabled(TRUE);
        cb_show_gap->setEnabled(FALSE);
        cb_qmap_reflect->setEnabled(TRUE);
        cb_show_sphere->setEnabled(TRUE);
        cb_qmap_rings->setEnabled(TRUE);
        gbox_graph_elements->show();

        if(mo->qshowAxes)
            cb_axes->setChecked(TRUE);
        else
            cb_axes->setChecked(FALSE);

        if(mo->qshowEdge)
            cb_edge->setChecked(TRUE);
        else
            cb_edge->setChecked(FALSE);

        if(mo->qshowFrame)
            cb_frames->setChecked(TRUE);
        else
            cb_frames->setChecked(FALSE);

        if(mo->qshowFace)
            cb_face->setChecked(TRUE);
        else
            cb_face->setChecked(FALSE);
		
        if(mo->qshowCurve)
            cb_show_curve->setChecked(TRUE);
        else
            cb_show_curve->setChecked(FALSE);

        if(mo->qshowReflectedPoints)
            cb_qmap_reflect->setChecked(TRUE);
        else
            cb_qmap_reflect->setChecked(FALSE);

        if(mo->qshowQuatRings)
            cb_qmap_rings->setChecked(TRUE);
        else
            cb_qmap_rings->setChecked(FALSE);
        
        if(mo->qshowSphere)
            cb_show_sphere->setChecked(TRUE);
        else
            cb_show_sphere->setChecked(FALSE);

        if(mo->selectFrame_FS)
            cb_qmap_f->setChecked(TRUE);
        
        switch(mo->qwhichQuatComponent) {
            case SHOWXYZ:
                rb_show_quatcomp_w->setChecked(TRUE);
                break;
            case SHOWWYZ:
                rb_show_quatcomp_x->setChecked(TRUE);
                break;
            case SHOWWXZ:
                rb_show_quatcomp_y->setChecked(TRUE);
                break;
            case SHOWWXY:
                rb_show_quatcomp_z->setChecked(TRUE);
                break;
            default:
                break;
        }
        
		if(mo->rshowFace)
			cb_qmap_face->setChecked(TRUE);
		else
			cb_qmap_face->setChecked(FALSE);

		if(mo->rshowEdge)
			cb_qmap_ring_edge->setChecked(TRUE);
		else
			cb_qmap_ring_edge->setChecked(FALSE);

        
    }
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2) {

        cb_show_curve->setText(QString("Rings (c)"));
        cb_edge->setEnabled(TRUE);
        cb_face->setText(QString("Surface (f)"));
        cb_frames->setEnabled(FALSE);
        cb_show_gap->setEnabled(FALSE);
        cb_qmap_reflect->setEnabled(FALSE);
        cb_show_sphere->setEnabled(FALSE);
		gbox_graph_elements->show();

        //if(mo->rshowAxes)
        //    cb_axes->setChecked(TRUE);
        //else
            cb_axes->setChecked(FALSE);

        //if(mo->rshowEdge) {
        //    cb_edge->setChecked(TRUE);
        //}
        //else
            cb_edge->setChecked(FALSE);

        //if(mo->rshowFrame)
        //    cb_frames->setChecked(TRUE);
        //else
            cb_frames->setChecked(FALSE);

        //if(mo->rshowFace)
        //    cb_face->setChecked(TRUE);
        //else
            cb_face->setChecked(FALSE);
		
        //if(mo->rshowCurve){
        //    cb_show_curve->setChecked(TRUE);
        //}
        //else
            cb_show_curve->setChecked(FALSE);

    }
	//GLOBALS
    if(mo->showLabels) 
        cb_show_labels->setChecked(TRUE);
    else
        cb_show_labels->setChecked(FALSE);


}
//# WINDOW EVENTS
void VisibleForm::closeEvent									( QCloseEvent *e ) 
{
   qApp->exit(0);
   //QMessageBox mb( "Exiting...",
   //                 "Really quit?",
   //                 QMessageBox::NoIcon,
   //                 QMessageBox::Yes | QMessageBox::Default,
   //                 QMessageBox::No,
   //                 QMessageBox::NoButton );
   // if(mb.exec() == QMessageBox::Yes) {
   //     e->accept();
   //     qApp->exit(0);
   // }
}

void VisibleForm::exitApp										( void ) 
{
    fprintf(stderr,"Normal exit\n");
    qApp->exit(0);
}

//# EVENT MGMT
void VisibleForm::updateForm									( void )
{
    int	i = 0;
	int limit = 0;
	if(mo->flag_curve_closed)
		limit = mo->numPoints-1;
	else
		limit = mo->numPoints-2;

    cbox_track_ring->clear();
    cbox_track_ring->insertItem(QString("None"));
    for(i=0; i<limit; i++) {
        cbox_track_ring->insertItem(QString(QString::number(i)));
    }
    if( (mo->trackingRing!= -1) && (mo->trackingRing < mo->numPoints) )
        cbox_track_ring->setCurrentItem(mo->trackingRing+1);
    else
        cbox_track_ring->setCurrentItem(0);

    cbox_surface_type->clear();
    for(i=0; i<mo->surfNumObjects; i++) {
        //cbox_surface_type->insertItem(QString(QString::number(i)));
		switch( i ) {

			case 0:
				cbox_surface_type->insertItem(QString("Torus")); break;
			case 1:
				cbox_surface_type->insertItem(QString("Sphere")); break;
			case 2:
				cbox_surface_type->insertItem(QString("Catenoid")); break;
			case 3:
				cbox_surface_type->insertItem(QString("Enneper's Surface")); break;
			case 4:
				cbox_surface_type->insertItem(QString("nothing...")); break;
			default:
				break;
		}
    }
    if( (mo->surfObjectID!= -1) && (mo->surfObjectID < mo->surfNumObjects) )
        cbox_surface_type->setCurrentItem(mo->surfObjectID);
    else
        cbox_surface_type->setCurrentItem(0);

    //cbox_palette->setCurrentItem(0);
    //mo->whichPalette = 0;

}
void VisibleForm::updateObject									( void )
{
	mo->computeObject();
}
void VisibleForm::updateQuaternionRingElements					( void )
{
	quaternion	*r;
	r =	(quaternion*) my_malloc(sizeof(quaternion));
	dial_quat_ring_angle->setValue(0);
	mo->rquatRingAngle = 0.0;
	//mo->findQuaternion( mo->trackingRing, r, &(mo->rtrackQuatAngle) );
    my_free(r);


}
void VisibleForm::calbak_ResetApp								( void ) 
{
    vp->resetApp();
    notifyOGLWidgets();
}

void VisibleForm::resizeEvent									( QResizeEvent * ) 
{
    
        //   int extraWidth  = width()- minimumWidth();
        //   int extraHeight = height()-minimumHeight();

	//frm_OGLScreen1->setGeometry( 10,10,680+extraWidth, 680+extraHeight );
	//vp->setGeometry ( frm_OGLScreen1->x()+1,frm_OGLScreen1->y()+1,678+extraWidth,678+extraHeight );
	//vp->resizeGL( vp->width(), vp->height() );
        //   //c->setGeometry(OGLFrame->x()+1,OGLFrame->y()+1,558+extraWidth, 548+extraHeight);
        //   //c->resizeGL(c->width(),c->height());
        //   //notifyOGLWidgets();

        //   extraWidth  = width()-prevWidth;
        //   extraHeight = height()-prevHeight;

        //   //controlBox->setGeometry(controlBox->x()+extraWidth,controlBox->y(),controlBox->width(),controlBox->height());
        //   
	////groupBox_ColorPalate->setGeometry(0,groupBox_ColorPalate->y()+extraHeight, groupBox_ColorPalate->width(),groupBox_ColorPalate->height());

        //   prevWidth = width();
        //   prevHeight= height();
}

void VisibleForm::notifyOGLWidgets								( void )
{
    if(vp!=NULL)
        vp->updateGL();
    if(fwin_vp != NULL)
        fwin_vp->updateGL();
    if(fwin2_vp != NULL) 
        fwin2_vp->updateGL();
}

void VisibleForm::notifyOGLFloatingWidget                       ( void )
{
    if(fwin_vp != NULL) 
        fwin_vp->updateGL();
}
void VisibleForm::calbak_Swap_Frames							( void )
{
    if(swapFrames == 0) {
        fwin_vp->reparent(frm_OGLScreen1, 0, QPoint(0,0), 1);
        fwin_vp->setGeometry(1,1,main_wind_height,main_wind_height);
        fwin_vp->setFocus();
		toolbox1->setCurrentIndex(1);

        vp->reparent(frm_OGLScreen2, 0, QPoint(0,0), 1);
        vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        fwin2_vp->reparent(frm_OGLScreen3, 0, QPoint(0,0), 1);
        fwin2_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        swapFrames = !swapFrames;
        swapFrames2 = 0;
        mo->whoCommandsMainWin = AUXY_WINDOW_1;
    }
    else {
        vp->reparent(frm_OGLScreen1, 0, QPoint(0,0), 1);
        vp->setGeometry(1,1,main_wind_height,main_wind_height);
        vp->setFocus();
		toolbox1->setCurrentIndex(0);

        fwin_vp->reparent(frm_OGLScreen2, 0, QPoint(0,0), 1);
        fwin_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        fwin2_vp->reparent(frm_OGLScreen3, 0, QPoint(0,0), 1);
        fwin2_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        swapFrames = !swapFrames;
        swapFrames2 = 0;
        mo->whoCommandsMainWin = MAIN_WINDOW;
    }
    stateChange();
    notifyOGLWidgets();
}

void VisibleForm::calbak_Swap_Frames_2							( void ) {
    if(swapFrames2 == 0) {
        fwin2_vp->reparent(frm_OGLScreen1, 0, QPoint(0,0), 1);
        fwin2_vp->setGeometry(1,1,main_wind_height,main_wind_height);
        fwin2_vp->setFocus();
		toolbox1->setCurrentIndex(2);

        vp->reparent(frm_OGLScreen3, 0, QPoint(0,0), 1);
        vp->setGeometry(1,1,auxy_wind_height, auxy_wind_height);
        fwin_vp->reparent(frm_OGLScreen2, 0, QPoint(0,0), 1);
        fwin_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        swapFrames2 = !swapFrames2;
        swapFrames = 0;
        mo->whoCommandsMainWin = AUXY_WINDOW_2;
        mo->qfont = GLUT_BITMAP_HELVETICA_12;
    }
    else {
        vp->reparent(frm_OGLScreen1, 0, QPoint(0,0), 1);
        vp->setGeometry(1,1,main_wind_height,main_wind_height);
        vp->setFocus();
		toolbox1->setCurrentIndex(0);

		fwin2_vp->reparent(frm_OGLScreen3, 0, QPoint(0,0), 1);
        fwin2_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        fwin_vp->reparent(frm_OGLScreen2, 0, QPoint(0,0), 1);
        fwin_vp->setGeometry(1,1,auxy_wind_height,auxy_wind_height);
        swapFrames2 = !swapFrames2;
        swapFrames = 0;
        mo->whoCommandsMainWin = MAIN_WINDOW;
        mo->qfont = GLUT_BITMAP_HELVETICA_10;
    }
    stateChange();
    notifyOGLWidgets();
}
void VisibleForm::calbak_DrawingOptions							( )
{
    if(rb_draw_curve->isChecked()) {
        mo->objectID = OBJCURVE;
        gbox_select_1->setEnabled(TRUE);
		gbox_select_2->setEnabled(FALSE);
		calbak_DrawingOptions_CurveType();
    }
    else if(rb_draw_surface->isChecked()) {
        mo->objectID = OBJSURFACE;
        mo->whichCurve = OBJSURFACE;
		mo->whichCurve=-1;
        if(mo->flag_surf_init) {
            mo->initStructs_Surfaces();
            mo->flag_surf_init = 0;
        }
		mo->clearAll();
        gbox_select_1->setEnabled(FALSE);
		gbox_select_2->setEnabled(TRUE);
		gbox_surface_1->show();
        gbox_curve_1->hide();
        gbox_curve_2->hide();
    }
	
    notifyOGLWidgets();
}
void VisibleForm::calbak_DrawingOptions_Palette() 
{
	mo->whichPalette = cbox_palette->currentItem();
    mo->updatePalette();
    updateObject();
    notifyOGLWidgets();
}
void VisibleForm::calbak_DrawingOptions_CurveType				( )
{
    mo->clearAll();
	if(rb_draw_curve_edit->isChecked()) {
        mo->whichCurve = OBJSPLINE;
        gbox_curve_1->hide();
        gbox_curve_2->show();
		gbox_surface_1->hide();
        cbox_curve_list->setEnabled(FALSE);
        mo->flag_curve_closed = 0;
    }
    else if(rb_draw_curve_knots->isChecked()) {
        mo->whichCurve = OBJKNOT;
        gbox_curve_1->show();
        gbox_curve_2->hide();
		gbox_surface_1->hide();
        cbox_curve_list->setEnabled(FALSE);
        mo->flag_curve_closed = 1;
		mo->computeObject();
    }
    else if(rb_draw_curve_list->isChecked()) {
        mo->whichCurve = OBJCURVELIST;
        cbox_curve_list->setEnabled(TRUE);
    }

	stateChange();
    notifyOGLWidgets();
}
void VisibleForm::calbak_DrawingOptions_CurveType_SplineType	( )
{
    if(rb_spline_bz->isChecked())
        mo->whichSpline = BEZIER;
    else if(rb_spline_bspline->isChecked())
        mo->whichSpline = BSPLINE;
    else if(rb_spline_catrom->isChecked())
        mo->whichSpline = CATROM;
    else 
        mo->whichSpline = NOSPLINE;
	
	//updateQuaternionRingElements();
	notifyOGLWidgets();

}
void VisibleForm::calbak_ViewControl_StatusBar                  ( )
{
    lb_status_text->setText(mo->status);
}
/* VIEW CONTROL CALLBACKS */
void VisibleForm::calbak_ViewControl_Frames						( void )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        mo->showFrame = !mo->showFrame;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        mo->qshowFrame = !mo->qshowFrame;
    notifyOGLWidgets();
}

void VisibleForm::calbak_ViewControl_Axes						( )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        mo->showAxes = !mo->showAxes;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        mo->qshowAxes = !mo->qshowAxes;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2)
        mo->rshowAxes = !mo->rshowAxes;
    notifyOGLWidgets();
}

void VisibleForm::calbak_ViewControl_Faces						( )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        mo->showFace = !mo->showFace;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        mo->qshowFace = !mo->qshowFace;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2)
        mo->rshowFace = !mo->rshowFace;

    notifyOGLWidgets();
}
void VisibleForm::calbak_ViewControl_EdgesOnly					( )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        mo->showEdge = !mo->showEdge;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        mo->qshowEdge = !mo->qshowEdge;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2) {
        mo->rshowEdge = !mo->rshowEdge;
        //mo->rshowCurve = 0;
    }
    notifyOGLWidgets();
}

void VisibleForm::calbak_ViewControl_Labels						( )
{
    mo->showLabels = !mo->showLabels;
    notifyOGLWidgets();
}

void VisibleForm::calbak_ViewControl_Palette					( )
{
	if(cb_palette->isChecked()) 
		mo->showPalette = 1;
	else
		mo->showPalette = 0;

	notifyOGLWidgets();
}
void VisibleForm::calbak_ViewControl_QMap_TubeThickness		( void )
{
    double val = 0.0;
    if(slider_qmap_tubethickness->value()<1)
        slider_qmap_tubethickness->setValue(1);
    
    val = ((double)slider_qmap_tubethickness->value());
    val = (1/Factor100)*val;
    
    mo->qmaptubeThickness = val;
	str.sprintf("%s %0.2f","Tube Thickness: ",val);
    lb_qmap_tubethickness->setText(str);

	updateObject();
    notifyOGLWidgets();
}
void VisibleForm::calbak_ViewControl_3DCurve_TubeThickness		( void )
{
    double val = 0.0;
    if(slider_curve_tubethickness->value()<1)
        slider_curve_tubethickness->setValue(1);
    
    val = ((double)slider_curve_tubethickness->value());
    val = (1/Factor100)*val;
    
    mo->tubeThickness = val;
	str.sprintf("%s %0.2f","Tube Thickness: ",val);
    lb_curve_tubethickness->setText(str);

	updateObject();
    notifyOGLWidgets();
}
/* Object CURVE CONTROL CALLBACKS */
void VisibleForm::calbak_SplineTransform_t						( )
{
    double val = ( (double)slider_spline_t->value() )/Factor100;
    str.sprintf( "%s %0.2f","t: ", val );
    lb_num_pnts->setText( str );
    mo->param_spline_increment = val;
    notifyOGLWidgets();
}
/* Object KNOT CONTROL CALLBACKS */
void VisibleForm::calbak_KnotTransform_Param_m					( void )
{
    if(slider_knot_m->value()<1)
        slider_knot_m->setValue(1);
    
    mo->m = slider_knot_m->value();
    str.sprintf("%s %d","m=",slider_knot_m->value());
    lb_knot_m->setText(str);
	
    mo->flag_update_track_ring = 1;
	updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_Param_n					( void )
{
    if(slider_knot_n->value()<1)
        slider_knot_n->setValue(1);    
    
    mo->n = slider_knot_n->value();
    str.sprintf("%s %d","n=",slider_knot_n->value());
    lb_knot_n->setText(str);

    mo->flag_update_track_ring = 1;
	updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_Param_a					( void )
{
    double val=mo->a;

    val = ((double)slider_knot_a->value());
    val = (1/Factor1000)*val;
    
    mo->a = val;
    str.sprintf("%s %0.3f","a=",val);
    lb_knot_a->setText(str);
    
    
    if(mo->lock_a_b) {
        str.sprintf("%s %0.3f","b=",val);
        lb_knot_b->setText(str);
        slider_knot_b->setValue(slider_knot_a->value());
        mo->b = val;
    }
	updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_Param_b					( void )
{
    double val=mo->b;
    
    val = ((double)slider_knot_b->value());
    val = (1/Factor1000)*val;

    mo->b = val;
    str.sprintf("%s %0.3f","b=",val);
    lb_knot_b->setText(str);
    
    
    if(mo->lock_a_b) {
        str.sprintf("%s %0.2f","a=",val);
        lb_knot_a->setText(str);
        slider_knot_a->setValue(slider_knot_b->value());
        mo->a = val;
    }
	updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_LockAB					( void )
{
    if( cb_lock_a_b->isChecked()) {
        mo->lock_a_b=1;
        mo->b = mo->a;
        str.sprintf("%s %0.2f","b=",mo->b);
        lb_knot_b->setText(str);
        slider_knot_b->setValue(slider_knot_a->value());
        notifyOGLWidgets();
    }
    else
        mo->lock_a_b=0;    
}


void VisibleForm::calbak_KnotTransform_Param_r					( void )
{
    double val=mo->r;
    
    val = ((double)slider_knot_radius->value());
    val = (1/Factor100)*val;

    mo->r = val;
    str.sprintf("%s %0.2f","r=",val);
    lb_knot_r->setText(str);
    
	updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_Param_startangle			( void )
{
    double val= ((double)slider_pt_start_angle->value());
	
    val = (1/Factor100)*val;
    if(val>4*M_PI)
        val=4*M_PI;
    if(val<0.0)
        val=4*M_PI;

    str.sprintf("%s %0.2f","PT: Start angle=",val);
    lb_knot_start_angle->setText(str);


	//update all copy sliders
    slider_pt_start_angle_2->setValue(slider_pt_start_angle->value());
    lb_knot_start_angle_2->setText(str);

    mo->startTheta = val;
    mo->flag_quat_movingquat = 1;
    mo->selectFrame_PT = 1;
    cb_qmap_pt->setChecked(TRUE);
	updateObject();
    
	//notifyOGLWidgets();

}
void VisibleForm::calbak_KnotTransform_Param_startangle_2		( void )
{
    double val= ((double)slider_pt_start_angle_2->value());
	//printf("in here...\n");
    val = (1/Factor100)*val;
    if(val>4*M_PI)
        val=4*M_PI;
    if(val<0.0)
        val=4*M_PI;

    str.sprintf("%s %0.2f","PT: Start angle=",val);
    lb_knot_start_angle_2->setText(str);


	//update all copy sliders
    slider_pt_start_angle->setValue(slider_pt_start_angle_2->value());
    lb_knot_start_angle->setText(str);

    mo->startTheta = val;
    mo->selectFrame_PT = 1;
    cb_qmap_pt->setChecked(TRUE);
	updateObject();
    notifyOGLWidgets();

}
void VisibleForm::calbak_KnotTransform_Frames					( void )
{
    if(rb_frame_f->isOn()) {
        mo->whichFrame = FRENETSERRET;
        mo->selectFrame_FS = 1;
        cb_qmap_f->setChecked(TRUE);
    }
    else if(rb_frame_pt->isOn()) {
        mo->whichFrame = PARALLELTRANSPORT;
        mo->selectFrame_PT = 1;
        cb_qmap_pt->setChecked(TRUE);
    }
    else if(rb_frame_geodesic->isOn()) {
        mo->whichFrame = GEODESICREFERENCE;
        mo->selectFrame_GR = 1;
        cb_qmap_gr->setChecked(TRUE);
    }
    else if(rb_frame_noframe->isOn())
        mo->whichFrame = NOFRAME;
	updateObject();
    notifyOGLWidgets();
    
}

void VisibleForm::calbak_KnotTransform_QuatMapFrames			( void )
{
    if(cb_qmap_f->isChecked())
        mo->selectFrame_FS = 1;
    else
        mo->selectFrame_FS = 0;

    if(cb_qmap_pt->isChecked())
        mo->selectFrame_PT = 1;
    else
        mo->selectFrame_PT = 0;

    if(cb_qmap_gr->isChecked())
        mo->selectFrame_GR = 1;
    else
        mo->selectFrame_GR = 0;

	updateObject();
    notifyOGLWidgets();
    
}    
void VisibleForm::calbak_KnotTransform_QuatFrames				( void )
{
    if(rb_frame_f_quat->isChecked()) {
        mo->whichQuatFrame = FRENETSERRET;
    }
    else if(rb_frame_pt_quat->isChecked()) {
        mo->whichQuatFrame = PARALLELTRANSPORT;
    }
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotControl_Gap						( )
{
    if(cb_show_gap->isChecked())
        mo->showGap = 1;
    else
        mo->showGap = 0;
    notifyOGLWidgets();
}
void VisibleForm::calbak_KnotControl_Curve						( )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        mo->showCurve = !mo->showCurve;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        mo->qshowCurve = !mo->qshowCurve;
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2) {
        mo->rshowCurve = !mo->rshowCurve;
        mo->rshowEdge = 0;
    }

    notifyOGLWidgets();
}
void VisibleForm::calbak_KnotControl_GoTwice					( )
{
    if(cb_qmap_reflect->isChecked())
        mo->qshowReflectedPoints = 1;
    else
        mo->qshowReflectedPoints = 0;
    notifyOGLWidgets();
}
void VisibleForm::calbak_KnotControl_EditQuatMap				( )
{
    mo->flag_edit_quatmap = !mo->flag_edit_quatmap;
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_QuatMap_ShowSphere		( )
{
    mo->qshowSphere = !mo->qshowSphere;
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_QuatMap_Rings			( )
{
	mo->qshowQuatRings = !mo->qshowQuatRings;
    notifyOGLWidgets();
}

void VisibleForm::calbak_KnotTransform_QuatMap_FullSpace_Mesh	( )
{
	mo->rshowEdge = !mo->rshowEdge;
    notifyOGLWidgets();
}
void VisibleForm::calbak_KnotTransform_QuatMap_Space_Surface	( ) 
{
	mo->rshowFace = !mo->rshowFace;
	notifyOGLWidgets();
}
void VisibleForm::calbak_KnotTransform_QuatComponent			( )
{
    if(rb_show_quatcomp_w->isChecked()) {
        mo->whichQuatComponent = SHOWXYZ;
        mo->qwhichQuatComponent = SHOWXYZ;
    }
    else if(rb_show_quatcomp_x->isChecked()) {
        mo->whichQuatComponent = SHOWWYZ;
        mo->qwhichQuatComponent = SHOWWYZ;
    }
    else if(rb_show_quatcomp_y->isChecked()) {
        mo->whichQuatComponent = SHOWWXY;
        mo->qwhichQuatComponent = SHOWWXY;
    }
    else if(rb_show_quatcomp_z->isChecked()) {
        mo->whichQuatComponent = SHOWWXZ;
        mo->qwhichQuatComponent = SHOWWXZ;
    }
    updateObject();
    notifyOGLWidgets();
}

void VisibleForm::calbak_QuatRings_Number						( )
{   
    mo->numOfRings = slider_ring_number->value();
    str.sprintf("%s %0.0f %","Number of Rings= ", (((double)mo->numOfRings)/4.0)*100.0 );
    lb_ring_number->setText(str);
    mo->rshowCurve = 1;
    mo->qshowQuatRings = 1;
    stateChange();
	//updateObject();
    notifyOGLWidgets();
}


void VisibleForm::calbak_TrackRing								( )
{
    if(cbox_track_ring->currentItem() == 0) {
        mo->trackingRing = -1;
            //mo->bhiderest = 0;
            //cb_belt_multiple_hide_rest->setChecked(FALSE);
    }
    else {
        mo->trackingRing = cbox_track_ring->currentItem()-1;
        if(!mo->rshowCurve) {
            mo->rshowCurve = 1;
            stateChange();
        }
    }
	//updateQuaternionRingElements();
	notifyOGLWidgets();
}
void VisibleForm::calbak_QuatRings_AngleChange					( void )
{
	double	val = 0.0;
	double	increment = 4.0*M_PI/ (dial_quat_ring_angle->maxValue());
	
	val = dial_quat_ring_angle->value() * increment;
    str.sprintf("%s %0.3f","Angle=",val);
    lb_dial_quat_ring_angle->setText(str);
	//printf("dial val = %d\n",dial_quat_ring_angle->value() );

	mo->rquatRingAngle = val;
	//mo->updateQuatRings();
	//quatCopy(&(mo->fsQuatList[mo->trackingRing]),&(mo->opQuatList[mo->trackingRing]));
	//mo->opt_INDEX++;
    //   if(mo->opt_INDEX == mo->numPoints-1)
    //       mo->opt_INDEX = 1;
    //   mo->optimization();
    mo->optPT_setLastQuat();
	notifyOGLWidgets();
	
}
/* Object SURFACE CONTROL CALLBACKS */
void VisibleForm::calbak_Surface_Param_U						( void )
{
    if(slider_surface_u->value()<1)
        slider_surface_u->setValue(1);
    
    mo->u = slider_surface_u->value();
    str.sprintf("%s %d","m=",slider_surface_u->value());
    lb_surface_u->setText(str);
	
    notifyOGLWidgets();
}
void VisibleForm::calbak_Surface_Param_V						( void )
{    
	if(slider_surface_v->value()<1)
        slider_surface_v->setValue(1);
    
    mo->v = slider_surface_v->value();
    str.sprintf("%s %d","m=",slider_surface_v->value());
    lb_surface_v->setText(str);
	
    notifyOGLWidgets();
}
void VisibleForm::calbak_Surface_Param_A						( void )
{
    double val=mo->param_a;

    val = ((double)slider_surface_param_a->value());
    val = (1/Factor1000)*val;
    
    mo->param_a = val;
    str.sprintf("%s %0.3f","a=",val);
    lb_surface_param_a->setText(str);
    
    notifyOGLWidgets();
}
void VisibleForm::calbak_Surface_Param_B						( void )
{
    double val=mo->param_b;

    val = ((double)slider_surface_param_b->value());
    val = (1/Factor1000)*val;
    
    mo->param_b = val;
    str.sprintf("%s %0.3f","b=",val);
    lb_surface_param_b->setText(str);
    
    notifyOGLWidgets();
}


void VisibleForm::calbak_Surface_Param_R						( void )
{
    double val=mo->param_r;
    
    val = ((double)slider_surface_param_r->value());
    val = (1/Factor100)*val;

    mo->param_r = val;
    str.sprintf("%s %0.2f","r=",val);
    lb_surface_r->setText(str);
    
    
    notifyOGLWidgets();
}
void VisibleForm::calbak_Surface_SelectType						( void )
{
	mo->surfObjectID = cbox_surface_type->currentItem();
	mo->u						= 20;
	mo->v						= 20;
	mo->param_a				= 0.5;
	mo->param_b				= 0.5;

	notifyOGLWidgets();
}
/* Optimizations */
void VisibleForm::calbak_Optimization_Start						( void )
{
	printf("Started...\n");
	mo->flag_opt_continue = 1;
    mo->pt_flag_terminate = 0;
	startTimer(1);
}

void VisibleForm::calbak_Optimization_Stop						( void )
{
	mo->flag_opt_continue = 0;
	printf("Stopped!\n");
}
void VisibleForm::calbak_Optimization_Update					( void )
{
	mo->flag_opt_first_time = 1;
	mo->trackingRing = -1;
	//mo->fixedindex = 1+(int) (((double)mo->numRingPoints)*rand()/(RAND_MAX+1.0));
	//printf(" fixed index = %d\n",mo->fixedindex);
	//updateObject();
    mo->opt_INDEX = 1;
    mo->pt_flag_first_time = 1;
    mo->pt_flag_terminate = 0;
    dial_quat_ring_angle->setValue(0);
    mo->rquatRingAngle = 0.0;
	mo->initOPTIM();
	notifyOGLWidgets();
}
void VisibleForm::calbak_Optimization_Increment_Inc				( void )
{
	if(mo->flag_keypress_SHIFT)
		mo->opt_EPSILON += 0.01;
	else if(mo->flag_keypress_CTRL)
		mo->opt_EPSILON += 0.1;
	else
		mo->opt_EPSILON += 0.001;
    if(mo->opt_EPSILON > 1.0)
		mo->opt_EPSILON = 1.0;
	str.sprintf("%s %.3f","= ", mo->opt_EPSILON);
    lb_opt_increment->setText(str);
	
}
void VisibleForm::calbak_Optimization_Increment_Dec				( void )
{
	if(mo->flag_keypress_SHIFT)
		mo->opt_EPSILON -= 0.01;
	else if(mo->flag_keypress_CTRL)
		mo->opt_EPSILON -= 0.1;
	else
		mo->opt_EPSILON -= 0.001;
    if(mo->opt_EPSILON < 0.001)
		mo->opt_EPSILON = 0.001;
	str.sprintf("%s %.3f","= ", mo->opt_EPSILON);
    lb_opt_increment->setText(str);
}

void VisibleForm::calbak_Optimization_SingleStep				( void )
{
	int i = 0;
	//if(mo->whichCurve == OBJSPLINE)
	//	i = mo->numPoints-3;
	//else if(mo->whichCurve == OBJKNOT)
	//	i = mo->numPoints-1;
	//mo->opt_INDEX += 1;
	//if(mo->opt_INDEX > i)
	//	mo->opt_INDEX = 1;
	//mo->trackingRing = mo->opt_INDEX;
	//updateQuaternionRingElements();
	//mo->optimization();
	//notifyOGLWidgets();

	i = mo->numPoints-1;
	mo->opt_INDEX += 1;
	if(mo->opt_INDEX> i)
		mo->opt_INDEX = 0;

    if(mo->pt_flag_terminate) {
        mo->flag_opt_continue = 0;
    }

    mo->rquatRingAngle += 0.25;
    
	mo->optimization();
	notifyOGLWidgets();
}

/* Texture Mgmt */
void VisibleForm::texUpdateList									( void )
{

	QStringList dirList;
	int			i = 0;

    printf("Loading Textures:\n");
	currDir = QDir::currentDirPath();
	currDir = currDir + QString("/textures");
	
	dir->setPath( currDir );
	if(!dir->exists()) {
		printf("Directory %s does not exist...\n",currDir.ascii() );
		return;
	}
	dir->setNameFilter("*.ppm");
	dirList = dir->entryList();
	
    if(dirList.count() <= 0) {
		printf("Directory %s does not have texture files...\n",currDir.ascii() );
		return;
	}
	cbox_loadtexture->clear();
	cbox_loadtexture->insertItem( QString("Select file...") );
	
    for(i=0;i < (int)dirList.count();i++) {
		printf("\t Found texture file: %s\n",dirList[i].ascii() );
		cbox_loadtexture->insertItem(dirList[i]);
	}
	cbox_loadtexture->setCurrentItem(0);
}
void VisibleForm::calbak_Texture_Load							( const QString& text )
{
	QString path = QString("textures/");
	path = path+text;
	const char *fname = path.ascii();
	if(!mo->texReadTextureFile( fname ) ) {
		printf("Texture loading failed.\n");
	}
	notifyOGLWidgets();
}
void VisibleForm::calbak_Texture_ControlParam					( void )
{
     double val = mo->texParam;
    
    val = ((double)slider_texture_control->value());
    val = (1/Factor10)*val;

    str.sprintf("%s %0.1f","Texture param=",val);
    lb_slider_texture_control->setText(str);
    
    mo->texParam = val;
    notifyOGLWidgets();
   
}
void VisibleForm::calbak_Texture_Toggle							( void )
{
	if(cb_texture->isChecked()) {
		mo->showTexture = 1;
		cbox_loadtexture->setEnabled(TRUE);
	}
	else {
		mo->showTexture = 0;
		cbox_loadtexture->setEnabled(FALSE);
	}

	notifyOGLWidgets();
}
void VisibleForm::calbak_DrawingOptions_ColorCode				( void )
{
	if(rb_color_curvature->isChecked()) 
		mo->showColorCoded = CODE_CURVATURE;
	else if(rb_color_torsion->isChecked())
		mo->showColorCoded = CODE_TORSION;
	else if(rb_color_solid->isChecked())
		mo->showColorCoded = CODE_SOLID_COLOR;

	updateObject();
	notifyOGLWidgets();
}
/* General */
void VisibleForm::calbak_Help									( void ) 
{
    help = new HelpWindow(home, ".", 0, "help viewer");
    help->show();
}

void VisibleForm::calbak_OnScreenText							( )
{
	mo->showOnScreenHelp = !mo->showOnScreenHelp;
	notifyOGLWidgets();
}
void VisibleForm::genHelpString									( void )
{

}

void VisibleForm::calbak_DumpScreen								( )
{
    if(mo->whoCommandsMainWin == MAIN_WINDOW)
        vp->saveScreenAsBMP ( );
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_1)
        fwin_vp->saveScreenAsBMP ( );
    else if(mo->whoCommandsMainWin == AUXY_WINDOW_2)
        fwin2_vp->saveScreenAsBMP ( );
}

void VisibleForm::calbak_Clear									( )
{
    mo->initObject();
    initForm();
    notifyOGLWidgets();
}



/* GRAPH */
void VisibleForm::calbak_Graph_cbox_Qdist						( void )
{
	if(cb_graph_qdist->isChecked())
		mo->gshowQdist = 1;
	else
		mo->gshowQdist = 0;
	notifyOGLWidgets();
}
void VisibleForm::calbak_Graph_cbox_Qdotprod					( void )
{
	if(cb_graph_qdotprod->isChecked())
		mo->gshowQdotprod = 1;
	else
		mo->gshowQdotprod = 0;
	notifyOGLWidgets();
}
void VisibleForm::calbak_Graph_cbox_Qtangacc					( void )
{
	if(cb_graph_qtangacc->isChecked())
		mo->gshowQtangacc = 1;
	else
		mo->gshowQtangacc = 0;
	notifyOGLWidgets();
}
void VisibleForm::calbak_Graph_cbox_Qcurvature					( void )
{
	if(cb_graph_qcurvature->isChecked())
		mo->gshowCcurvature = 1;
	else
		mo->gshowCcurvature = 0;
	notifyOGLWidgets();
}
void VisibleForm::calbak_Graph_cbox_Qtorsion					( void )
{
	if(cb_graph_qtorsion->isChecked())
		mo->gshowCtorsion = 1;
	else
		mo->gshowCtorsion = 0;
	notifyOGLWidgets();
}

/* LOADING MESHVIEW FILE */
void VisibleForm::curvesUpdateList									( void )
{

	QStringList dirList;
	int			i = 0;

    printf("Loading List of Curves:\n");
	currDir = QDir::currentDirPath();
	currDir = currDir + QString("/data");
	
	dir->setPath( currDir );
	if(!dir->exists()) {
		printf("Directory %s does not exist...\n",currDir.ascii() );
		return;
	}
	dir->setNameFilter("*.line *.lines");
	dirList = dir->entryList();
	
    if(dirList.count() <= 0) {
		printf("Directory %s does not have list-of-curve files...\n",currDir.ascii() );
		return;
	}
	cbox_curve_list->clear();
	cbox_curve_list->insertItem( QString("Select file...") );
	for(i=0;i < (int)dirList.count();i++) {
		printf("\t Found MeshView file: %s\n",dirList[i].ascii() );
		cbox_curve_list->insertItem(dirList[i]);
	}
	cbox_curve_list->setCurrentItem(0);
}
void VisibleForm::calbak_CurveList_Load                         ( const QString & text ) 
{
	QString path = QString("data/");
	path = path+text;
	const char *fname = path.ascii();
	mo->loadData( fname );
	notifyOGLWidgets();
}
