#include "math3D.h"
#include <math.h>
#include <assert.h>
//////////////////////////////////////////////////
// MATH and TUBING Utilities
//////////////////////////////////////////////////

/*** length of a 3D vector */
double length3D(const double *pt1)
{
        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(pt1);
#endif

    return (sqrt(pt1[0]*pt1[0]+pt1[1]*pt1[1]+pt1[2]*pt1[2]));
}

/*** two vectors' difference is another vector ***/
void minus3D(const double *vect1, const double *vect2, double *vect3)
{
        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(vect1);
    assert(vect2);
    assert(vect3);
#endif

    vect3[0] = vect1[0] - vect2[0];
    vect3[1] = vect1[1] - vect2[1];
    vect3[2] = vect1[2] - vect2[2];
}

/*** calculate the dot product of two vectors ***/
double dot3D(const double *vect1, const double *vect2)
{
        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(vect1);
    assert(vect2);
#endif

    return (vect1[0]*vect2[0] + vect1[1]*vect2[1] + vect1[2]*vect2[2]);
}


/*** calculate the plane cross product : v cross w = r ***/
void cross3D(const double *v, const double *w, double *r)
{
        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(v);
    assert(w);
    assert(r);
#endif

    r[0] = v[1]*w[2] - v[2]*w[1];
    r[1] = v[2]*w[0] - v[0]*w[2];
    r[2] = v[0]*w[1] - v[1]*w[0];
}

void multMatVec(const double *v, const double *m, double *r) 
{
    r[0] = m[0]*v[0] + m[1]*v[1] + m[2]*v[2];
    r[1] = m[1]*v[0] + m[5]*v[1] + m[9]*v[2];
    r[2] = m[2]*v[0] + m[6]*v[1] + m[10]*v[2];
}

void initVec(double *v) 
{
    v[0] = 0.0;
    v[1] = 0.0;
    v[2] = 0.0;
}

void copyVec(const double *v1, double *v2) 
{
    v2[0] = v1[0];
    v2[1] = v1[1];
    v2[2] = v1[2];
}


void multScalarVec(const double d, const double *v, double *r)
{
    r[0] = d*v[0];
    r[1] = d*v[1];
    r[2] = d*v[2];
}

/*** unify a vector ***/
int unify3D(double *vect)
{
    double length;

        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(vect);
#endif

    length = sqrt(vect[0]*vect[0]+vect[1]*vect[1]+vect[2]*vect[2]);

    if (length == 0.0)
	return 0;

    vect[0] = vect[0] / length;
    vect[1] = vect[1] / length;
    vect[2] = vect[2] / length;

    return 1;
}

/*** copy vect1 from vect2 ***/
void copy3D(double *vect1, const double *vect2)
{
        // no assertion in math3D.c to keep the program efficiency
#ifdef _DEBUG
    assert(vect1);
    assert(vect2);
#endif

    vect1[0] = vect2[0];
    vect1[1] = vect2[1];
    vect1[2] = vect2[2];
}
// angle is in radian

void
rotate3D_3(double *inOutVec, const double *axis, const double angle)
{
    double c,s,outVec[3],mat[9],myaxis[3];

        // form the rotation matrix
    c = cos(angle);
    s = sin(angle);

    copy3D(myaxis,axis);
    unify3D(myaxis);

#define n1  (myaxis[0])
#define n2  (myaxis[1])
#define n3  (myaxis[2])

    mat[0] = c + n1*n1*(1-c);
    mat[1] = n1*n2*(1-c) - s*n3;
    mat[2] = n3*n1*(1-c) + s*n2;

    mat[3] = n1*n2*(1-c) + s*n3;
    mat[4] = c + n2*n2*(1-c);
    mat[5] = n3*n2*(1-c) - s*n1;

    mat[6] = n3*n1*(1-c) - s*n2;
    mat[7] = n3*n2*(1-c) + s*n1;
    mat[8] = c + n3*n3*(1-c);

        // perform rotation
    outVec[0] = dot3D(mat,  inOutVec);
    outVec[1] = dot3D(mat+3,inOutVec);
    outVec[2] = dot3D(mat+6,inOutVec);

    copy3D(inOutVec,outVec);
}
